@@ -2,7 +2,7 @@
 /* -----------------------------------------------------------------------------------------------------------
 Software License for The Fraunhofer FDK AAC Codec Library for Android
 
- Copyright  1995 - 2012 Fraunhofer-Gesellschaft zur Frderung der angewandten Forschung e.V.
+ Copyright  1995 - 2015 Fraunhofer-Gesellschaft zur Frderung der angewandten Forschung e.V.
   All rights reserved.
 
  1.    INTRODUCTION
@@ -116,11 +116,6 @@ static const FIXP_DBL fadeOutFactor[FADE_OUT_LEN] = {1840644096, 1533870080, 122
 /* forward definitions */
 
 
-static inline int isLowDelay( AUDIO_OBJECT_TYPE aot )
-{
-  return (aot==AOT_ER_AAC_LD || aot==AOT_ER_AAC_ELD);
-}
-
 /*****************************************************************************
 
     functionname: FDKaacEnc_PsyNew
@@ -347,6 +342,7 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMainInit(PSY_INTERNAL *hPsy,
         tnsChannels,
         LONG_WINDOW,
         hPsy->granuleLength,
+        isLowDelay(audioObjectType),
         (syntaxFlags&AC_SBR_PRESENT)?1:0,
        &(hPsy->psyConf[0].tnsConf),
        &hPsy->psyConf[0],
@@ -367,6 +363,7 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMainInit(PSY_INTERNAL *hPsy,
             tnsChannels,
             SHORT_WINDOW,
             hPsy->granuleLength,
+            isLowDelay(audioObjectType),
             (syntaxFlags&AC_SBR_PRESENT)?1:0,
            &hPsy->psyConf[1].tnsConf,
            &hPsy->psyConf[1],
@@ -452,7 +449,7 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
                                     INT                  totalChannels
                                    )
 {
-    INT commonWindow = 1;
+    const INT commonWindow = 1;
     INT maxSfbPerGroup[(2)];
     INT mdctSpectrum_e;
     INT ch;   /* counts through channels          */
@@ -513,28 +510,28 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
 
       for(ch = 0; ch < channels; ch++)
       {
-          C_ALLOC_SCRATCH_START(timeSignal, INT_PCM, (1024));
-          psyStatic[ch]->blockSwitchingControl.timeSignal = timeSignal;
+          C_ALLOC_SCRATCH_START(pTimeSignal, INT_PCM, (1024))
 
           /* deinterleave input data and use for block switching */
-          FDKaacEnc_deinterleaveInputBuffer( psyStatic[ch]->blockSwitchingControl.timeSignal,
+          FDKaacEnc_deinterleaveInputBuffer( pTimeSignal,
                                             &pInput[chIdx[ch]],
                                              psyConf->granuleLength,
                                              totalChannels);
 
 
           FDKaacEnc_BlockSwitching (&psyStatic[ch]->blockSwitchingControl,
-                                     psyConf->granuleLength
-                                    ,psyStatic[ch]->isLFE
+                                     psyConf->granuleLength,
+                                     psyStatic[ch]->isLFE,
+                                     pTimeSignal
                                    );
 
 
             /* fill up internal input buffer, to 2xframelength samples */
             FDKmemcpy(psyStatic[ch]->psyInputBuffer+blockSwitchingOffset,
-                      psyStatic[ch]->blockSwitchingControl.timeSignal,
+                      pTimeSignal,
                       (2*psyConf->granuleLength-blockSwitchingOffset)*sizeof(INT_PCM));
 
-            C_ALLOC_SCRATCH_END(timeSignal, INT_PCM, (1024));
+            C_ALLOC_SCRATCH_END(pTimeSignal, INT_PCM, (1024))
       }
 
       /* synch left and right block type */
@@ -626,7 +623,7 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
           FDKmemclear(&psyData[ch]->mdctSpectrum[psyData[ch]->lowpassLine+wOffset],
                       (windowLength[ch]-psyData[ch]->lowpassLine)*sizeof(FIXP_DBL));
 
-          if (hPsyConfLong->filterbank != FB_LC) {
+          if ( (hPsyConfLong->filterbank != FB_LC) && (psyData[ch]->lowpassLine >= FADE_OUT_LEN) ) {
             /* Do blending to reduce gibbs artifacts */
             for (int i=0; i<FADE_OUT_LEN; i++) {
               psyData[ch]->mdctSpectrum[psyData[ch]->lowpassLine+wOffset - FADE_OUT_LEN + i] = fMult(psyData[ch]->mdctSpectrum[psyData[ch]->lowpassLine+wOffset - FADE_OUT_LEN + i], fadeOutFactor[i]);
@@ -768,7 +765,8 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
 
     /* Advance psychoacoustics: Tonality and TNS */
     if (psyStatic[0]->isLFE) {
-        tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive = 0;
+        tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive[HIFILT] = 0;
+        tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive[LOFILT] = 0;
     }
     else
     {
@@ -820,15 +818,19 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
                       &hThisPsyConf[1]->tnsConf);
             }
 
-            FDK_ASSERT(commonWindow=1); /* all checks for TNS do only work for common windows (which is always set)*/
+            FDK_ASSERT(1==commonWindow); /* all checks for TNS do only work for common windows (which is always set)*/
             for(w = 0; w < nWindows[0]; w++)
             {
                 if (isShortWindow[0])
-                    tnsActive[w] = tnsData[0]->dataRaw.Short.subBlockInfo[w].tnsActive ||
-                    ((channels == 2) ? tnsData[1]->dataRaw.Short.subBlockInfo[w].tnsActive : 0);
+                    tnsActive[w] = tnsData[0]->dataRaw.Short.subBlockInfo[w].tnsActive[HIFILT] ||
+                    tnsData[0]->dataRaw.Short.subBlockInfo[w].tnsActive[LOFILT] ||
+                    tnsData[channels-1]->dataRaw.Short.subBlockInfo[w].tnsActive[HIFILT] ||
+                    tnsData[channels-1]->dataRaw.Short.subBlockInfo[w].tnsActive[LOFILT];
                 else
-                    tnsActive[w] = tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive ||
-                    ((channels == 2) ? tnsData[1]->dataRaw.Long.subBlockInfo.tnsActive : 0);
+                    tnsActive[w] = tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive[HIFILT] ||
+                    tnsData[0]->dataRaw.Long.subBlockInfo.tnsActive[LOFILT] ||
+                    tnsData[channels-1]->dataRaw.Long.subBlockInfo.tnsActive[HIFILT] ||
+                    tnsData[channels-1]->dataRaw.Long.subBlockInfo.tnsActive[LOFILT];
             }
 
             for(ch = 0; ch < channels; ch++) {
@@ -1155,8 +1157,8 @@ AAC_ENCODER_ERROR FDKaacEnc_psyMain(INT                 channels,
                        psyData[ch]->sfbMaxScaleSpec.Long,
                        sfbTonality[ch],
                        psyOutChannel[ch]->tnsInfo.order[0][0],
-                       tnsData[ch]->dataRaw.Long.subBlockInfo.predictionGain,
-                       tnsData[ch]->dataRaw.Long.subBlockInfo.tnsActive,
+                       tnsData[ch]->dataRaw.Long.subBlockInfo.predictionGain[HIFILT],
+                       tnsData[ch]->dataRaw.Long.subBlockInfo.tnsActive[HIFILT],
                        psyOutChannel[ch]->sfbEnergyLdData,
                        psyOutChannel[ch]->noiseNrg );
         } /* !isLFE */
@@ -1345,7 +1347,7 @@ void FDKaacEnc_PsyClose(PSY_INTERNAL   **phPsyInternal,
 
       if (hPsyInternal)
       {
-        for (i=0; i<(6); i++) {
+        for (i=0; i<(8); i++) {
           if (hPsyInternal->pStaticChannels[i]) {
             if (hPsyInternal->pStaticChannels[i]->psyInputBuffer)
               FreeRam_aacEnc_PsyInputBuffer(&hPsyInternal->pStaticChannels[i]->psyInputBuffer);  /* AUDIO INPUT BUFFER */
@@ -1354,7 +1356,7 @@ void FDKaacEnc_PsyClose(PSY_INTERNAL   **phPsyInternal,
           }
         }
 
-        for (i=0; i<(6); i++) {
+        for (i=0; i<(8); i++) {
           if (hPsyInternal->psyElement[i])
             FreeRam_aacEnc_PsyElement(&hPsyInternal->psyElement[i]);                             /* PSY_ELEMENT */
         }
@@ -1368,12 +1370,12 @@ void FDKaacEnc_PsyClose(PSY_INTERNAL   **phPsyInternal,
       for (n=0; n<(1); n++) {
         if (phPsyOut[n])
         {
-          for (i=0; i<(6); i++) {
+          for (i=0; i<(8); i++) {
             if (phPsyOut[n]->pPsyOutChannels[i])
               FreeRam_aacEnc_PsyOutChannel(&phPsyOut[n]->pPsyOutChannels[i]);                  /* PSY_OUT_CHANNEL */
           }
 
-          for (i=0; i<(6); i++) {
+          for (i=0; i<(8); i++) {
             if (phPsyOut[n]->psyOutElement[i])
               FreeRam_aacEnc_PsyOutElements(&phPsyOut[n]->psyOutElement[i]);                   /* PSY_OUT_ELEMENTS */
           }
